/*///////////////////////////////Z80MicroSystem Emulator V0.2/////////////////////////////
//////////////////////////////by Del Nero Jean Francois///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////*/

#include <stdio.h>

#include "Z80.h"


// Liste des port d`IO
#define PIO1A 0
#define PIO1B 1
#define PIO1C 2
#define PIO1D 3

#define PIO2A 4
#define PIO2B 5
#define PIO2C 6
#define PIO2D 7

#define UART0 8
#define UART1 9
#define UART2 10
#define UART3 11
#define UART4 12
#define UART5 13
#define UART6 14
#define UART7 15

#define RTCADR 17
#define RTCIO 16

typedef struct {
	char PIOA;
	char PIOB;
	char PIOC;
	char PIOCONT;
}Un8255;

typedef struct {
	char RUART0;
	char RUART1;
	char RUART2;
	char RUART3;
	char RUART4;
	char RUART5;
	char RUART6;
	char RUART7;
}Un16455;

typedef struct {
	char RRTCADR;
	char RRTCIO;
	char MEMRTC[256];
}UnRTC;

word ReadRTC(UnRTC* thisrtc)
{
return thisrtc->MEMRTC[thisrtc->RRTCADR];
}

void WriteRTC(UnRTC* thisrtc,char Value)
{
thisrtc->MEMRTC[thisrtc->RRTCADR]=Value;
}

char Rom[16*1024];
char Ram[32*1024];


Z80 leZ80;
Un16455 Le16455;
Un8255 Le8255n1;
Un8255 Le8255n2;
UnRTC LeRTC;


	
/** RdZ80()/WrZ80() ******************************************/
/** These functions are called when access to RAM occurs.   **/
/** They allow to control memory access.                    **/
/************************************ TO BE WRITTEN BY USER **/
void WrZ80(register word Addr,register byte Value){
	if((Addr>=16*1024) && (Addr<48*1024)){
		Ram[Addr-16*1024]=Value;
	return;
	};
};

byte RdZ80(register word Addr){

	if((Addr>=16*1024) && (Addr<48*1024)){
	return Ram[Addr-16*1024];
	}
	
	if(Addr<16*1024)
	{
		//printf("%x",Rom[Addr]);
		return Rom[Addr];
	}
	
	return 0;
};


/** InZ80()/OutZ80() *****************************************/
/** Z80 emulation calls these functions to read/write from  **/
/** I/O ports. There can be 65536 I/O ports, but only first **/
/** 256 are usually used                                    **/
/************************************ TO BE WRITTEN BY USER **/
void OutZ80(register word Port,register byte Value){
	if (Port==UART0) printf("%c",Value);
	if (Port==RTCADR) LeRTC.RRTCADR=Value;
	if (Port==RTCIO) WriteRTC(&LeRTC,Value);

};
byte InZ80(register word Port){
	if (Port==UART0) return Le16455.RUART0;
	if (Port==UART1) return Le16455.RUART1;
	if (Port==UART2) return Le16455.RUART2;
	if (Port==UART3) return Le16455.RUART3;
	if (Port==UART4) return Le16455.RUART4;
	if (Port==UART5) return 255;//Le16455.RUART5;
	if (Port==UART6) return Le16455.RUART6;
	if (Port==UART7) return Le16455.RUART7;
	
	if (Port==RTCADR) return LeRTC.RRTCADR;
	if (Port==RTCIO) return ReadRTC(&LeRTC);

	return 0;};

/** PatchZ80() ***********************************************/
/** Z80 emulation calls this function when it encounters a  **/
/** special patch command (ED FE) provided for user needs.  **/
/** For example, it can be called to emulate BIOS calls,    **/
/** such as disk and tape access. Replace it with an empty  **/
/** macro for no patching.                                  **/
/************************************ TO BE WRITTEN BY USER **/
void PatchZ80(register Z80 *R){};

/** LoopZ80() ************************************************/
/** Z80 emulation calls this function periodically to check **/
/** if the system hardware requires any interrupts. This    **/
/** function must return an address of the interrupt vector **/
/** (0x0038, 0x0066, etc.) or INT_NONE for no interrupt.    **/
/** Return INT_QUIT to exit the emulation loop.             **/
/************************************ TO BE WRITTEN BY USER **/
word LoopZ80(register Z80 *R){

	if (kbhit()){
		char c=getch();
		if(c=='/') return INT_QUIT;
		
		Le16455.RUART0=c;
		Le16455.RUART2=0;
		return 0x38;
	};
			
return INT_NONE;};



FILE* lefichier; 



void main()
{

 lefichier=fopen("z80sys.bin","r");
 if(lefichier==NULL){
	 printf("Erreur de lecture de la ROM !!\n");
	 return;
 }
 
 if(fread(Rom,16*1024,1,lefichier)==16*1024)
 {
	 printf("Rom Ok....\n");
 };
 
 fclose(lefichier);

 printf(" Z80SYS EMULATOR\n ");
 
 printf(" reset Z80 CPU\n ");
 ResetZ80(&leZ80);
 printf(" Start Z80 CPU \n");
 RunZ80(&leZ80);
 
 printf("\n OK Z80Sys Off .\n ");

}
